# License:
#   Copyright (C) 2011 Thomas GUILLET, Damien CHAPON, Marc LABADENS. All Rights Reserved.
#
#   This file is part of PyMSES.
#
#   PyMSES is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   PyMSES is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with PyMSES.  If not, see <http://www.gnu.org/licenses/>.
#!/usr/bin/env python
# -*- coding: utf-8 -*-
# transfer function ray tracing map script
import pymses, os
from pymses.analysis.visualization import *
from pymses.analysis.visualization.image_plot_utils import *
from time import time
from pymses.analysis.visualization.raytracing import OctreeRayTracer, RayTracer
from pymses.sources.ramses import CameraOctreeDatasource
from optparse import OptionParser
from numpy import log10
t0 = time()
parser = OptionParser()
parser.usage = "%prog ramses_directory ramses_output_number map_max_resolution=512"
(opts, args) = parser.parse_args()
try:
	fileDir = args[0]
	outNumber = int(args[1])
	mms = int(args[2])
except:
	fileDir = None
	outNumber = None
	mms = 512
ro = pymses.RamsesOutput(fileDir, outNumber)
outNumber = ro.iout
cam = Camera(center=[ 0.5, 0.5, 0.5 ], line_of_sight_axis="z", up_vector="y", region_size=[5.0E-1, 5.0E-1], \
			distance=2.5E-1, far_cut_depth=2.5E-1, map_max_size=mms)
#import tables
#file= tables.openFile("camera.h5", "r")
#cam = Camera.from_HDF5(file)
#file.close()
cltf = ColorLinesTransferFunction( (-5.0, 2.0) )
cltf.add_line(-2.0, 0.1)
cltf.add_line(.0, 0.1)
cltf.add_line(2., 0.1)
cam.set_color_transfer_function(cltf)
# We add 1e-8 to avoid NaN and -Inf log result problems with approximative null values
op = ScalarOperator(lambda dset: log10(dset["rho"]+1e-8))

source = ro.amr_source(["rho"])
esize = 0.5**(ro.info["levelmin"]+1)
fullOctreeDataSource = CameraOctreeDatasource(cam, esize, source).dset
#rt = OctreeRayTracer(ro, ["rho"])
rt = OctreeRayTracer(fullOctreeDataSource)
img=rt.process(op, cam)
#img.show()
img.save("rt_tf_%s.png"%outNumber)
print "rt_tf_%s.png saved"%outNumber
#op = ScalarOperator(lambda dset: dset["rho"])
#rt = RayTracer(ro, ["rho"])
#map=rt.process(op, cam, source = fullOctreeDataSource, use_hilbert_domain_decomp = False)
#save_map_HDF5(map, cam, map_name="rt_%s"%outNumber)
#save_HDF5_to_img(("./rt_%s.h5"%outNumber), cmap="jet",img_path="./")
#os.remove(("./rt_%s.h5"%outNumber))
t1=time()
print "rt total time = %.1f s"%(t1-t0), "mms = ", mms, "max AMR read = ", cam.get_required_resolution()