/******************************************************************************************
* License:                                                                                *
*   Copyright (C) 2011 Thomas GUILLET, Damien CHAPON, Marc LABADENS. All Rights Reserved. *
*                                                                                         *
*   This file is part of PyMSES.                                                          *
*                                                                                         *
*   PyMSES is free software: you can redistribute it and/or modify                        *
*   it under the terms of the GNU General Public License as published by                  *
*   the Free Software Foundation, either version 3 of the License, or                     *
*   (at your option) any later version.                                                   *
*                                                                                         *
*   PyMSES is distributed in the hope that it will be useful,                             *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of                        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                         *
*   GNU General Public License for more details.                                          *
*                                                                                         *
*   You should have received a copy of the GNU General Public License                     *
*   along with PyMSES.  If not, see <http://www.gnu.org/licenses/>.                       *
******************************************************************************************/
#include"ray_trace_C_code.h"

static void ray_trace_C(double * maps, double * ray_length, double * ray_origins, double * ray_vects,
	double * cell_centers, int * sons, int * g_levels, double * scal_data,
	int * igrids, char * icells, long * active_mask, double * ray_depth_max,
	long * param_info_int){
//	printf("ray_trace_C ! \n");
//	int j =0;
//	for (j =0;j<16;j++){
//		printf(" %i ",j);
//		printf("active_mask = %i \n",active_mask[j]);
//	}
	int nrays = param_info_int[0];
	int ndim = param_info_int[1];
	int nblocks = param_info_int[2];
	int max_read_level = param_info_int[3];
	int nfunc = param_info_int[4];
	int max_op = param_info_int[5];
	int use_dx = param_info_int[6];
	int ngrids = param_info_int[7];
	
	int twotondim = 1 << ndim;
	int twotondimMndim = twotondim * ndim;
	int twotondimMnfunc = twotondim * nfunc;
	int igrid_root, level,iblock,idim,idim2,iray,head;
	int igrid, icell, igrid_son, ilevel, ifunc;
	int ind_son, i, intersect, sgn_face, block_already_done;
	int ind_son1, ind_son2, ind_entry, ind_exit, add, iblock2;
	char icell_root;
	double alpha, alpha_save, dx, dx_2, cube_depth, r2, l, d, v, vl;
	double Rsphere, Rsphere2;
	double sqrt3 = sqrt(3.0);
	double dm2, depth_max;
//	printf("param_info_int[7] = %i \n",ngrids);

	int* igrid_pile = NULL;
	int* icell_pile = NULL;
	int* level_pile = NULL;
	int* dim_constr = NULL;
	int* dim_side = NULL;
	igrid_pile = malloc((twotondim-1) * max_read_level * sizeof(int));
	icell_pile = malloc((twotondim-1) * max_read_level * sizeof(int));
	level_pile = malloc((twotondim-1) * max_read_level * sizeof(int));
	dim_constr = malloc(ndim * sizeof(int));
	dim_side = malloc(ndim * sizeof(int));
	
	double* block_center = NULL;
	block_center = malloc(ndim * sizeof(double));
	
	for (iblock = 0 ; iblock < nblocks ; iblock++){
		igrid_root = igrids[iblock];
		icell_root = icells[iblock];
		block_already_done = 0;
		for (iblock2 = 0 ; iblock2 < iblock ; iblock2++){
			if ((igrids[iblock2] == igrids[iblock]) && (icells[iblock2] == icells[iblock])){
				block_already_done = 1;
				break;
			}
		}
		if (block_already_done == 1)
			continue;
		level = g_levels[igrid_root];
		dx = 1.0/(1<<level);
		Rsphere = sqrt3 * dx / 2.0;
		Rsphere2 = Rsphere*Rsphere;
		for (idim = 0 ; idim < ndim ; idim++){
			block_center[idim] = cell_centers[igrid_root * twotondimMndim + icell_root * ndim+ idim];
		}
//		#################
//		# Ray iteration #
//		#################
		for (iray = 0 ; iray < nrays ; iray++){
			depth_max = ray_depth_max[iray];
			dm2 = depth_max/2.0;
//			# Coordinate parameter of the point of the ray nearest to the block center
			alpha = 0.0;
			for (idim = 0 ; idim < ndim ; idim++){
				alpha += (block_center[idim]-ray_origins[iray * ndim + idim])*ray_vects[iray * ndim + idim];
			}
//			# square distances of the ray from the block center
			r2 = 0.0;
			for (idim = 0 ; idim < ndim ; idim++){
				v = ray_origins[iray * ndim + idim] + alpha * ray_vects[iray * ndim + idim] - block_center[idim];
				r2 += v*v;
			}
//			# Is this ray able to intersect the block ?
			if (r2 > Rsphere2)// # no => continue
				continue;
//			# distance from the ray extreme points
			d = alpha-dm2;
			if (d<0.)
				d=-d;
			d = d-dm2;
			if (d>0.) {
				r2 += d*d;
//				# Is this ray still able to intersect the block ?
				if (r2 > Rsphere2)
					continue;
			}
//			# Pile initialisation with current block
			head=0;
			igrid_pile[head] = igrid_root;
			icell_pile[head] = icell_root;
			level_pile[head] = level;
			while (head !=-1) {
				igrid = igrid_pile[head];
				icell = icell_pile[head];
				ilevel = level_pile[head];
				head -= 1;
/*				printf("head = %i ",head);*/
/*				printf("igrid = %i ",igrid);*/
/*				printf("icell = %i ",icell);*/
/*				printf("ilevel = %i \n",ilevel);*/
//				# Current cell size
				dx = 1.0/(1<<ilevel);
				dx_2 = dx/2;
//				# Cube optical depth computation
				cube_depth = 0.0;
				intersect = 0;
				for (idim2 = 0 ; idim2 < ndim ; idim2++){
					if(ray_vects[iray * ndim + idim2] == 0.0){
						// ray is parallel to the face along dimension idim.
						// There might not be an intersection.
						continue;
					}
					// All ray-cell intersections found => break loop
					if (intersect==2) break;
					for (sgn_face = -1 ; sgn_face < 2 ; sgn_face+=2){ // for sgn_face in [-1,1]:
//						# Coordinate parameter of the intersection of the ray with the face plane
						alpha = ((cell_centers[igrid * twotondimMndim + icell * ndim + idim2] + sgn_face * dx / 2.0) - ray_origins[iray * ndim + idim2])/ray_vects[iray * ndim + idim2];
//							# Is this intersection within the face ?
						v=0.0;
						vl=0.0;
						ind_son = (twotondim-1) - (((-sgn_face+1)>>1)<<idim2);
						for (idim = 0 ; idim < ndim ; idim++){
							if (idim != idim2) {
								l = (ray_origins[iray * ndim + idim] + alpha * ray_vects[iray * ndim + idim] - cell_centers[igrid * twotondimMndim + icell * ndim + idim])/dx;
								if ((l>vl) && (ray_vects[iray * ndim + idim] == 0.0))
									vl=l;
								if (l<0.0){
									l=-l;
									ind_son -= (1<<idim);
								}
								if (l>v)
									v=l;
							}
						}
						if ((v <= 0.5) && (vl < 0.5)){
//							# The intersection is in the face of the cube
							intersect +=1;
							if (intersect == 1){ //# First intersection found
								if (alpha < 0.0)
									alpha_save = 0.0;
								else if (alpha > depth_max)
									alpha_save = depth_max;
								else
									alpha_save = alpha;
								ind_son1=ind_son;
							}
							else{// # Second intersection found : done
								if (alpha < 0.0)
									alpha = 0.0;
								else if (alpha > depth_max)
									alpha = depth_max;
								cube_depth = alpha-alpha_save;
//									# Make sure ind_son1 = entry point son index
//									#           ind_son2 = exit point son index
								if (cube_depth < 0.0) {// # ind_son is the entry point son index
									cube_depth=-cube_depth;
									ind_son2 = ind_son1;
									ind_son1 = ind_son;
								}
								else//# ind_son is the exit point son index
									ind_son2=ind_son;
								break;
							}
						}
					}
				}
//				# Is the current cell actually intersected by the ray ?
				if (cube_depth > 0.0){
//					# The ray cuts the block
					igrid_son = sons[igrid * twotondim + icell];
					if ((igrid_son < 0) || (ilevel >= max_read_level)){
//					# We're done searching: the subcell is the one we want
//						# We skip inactive cells
						if (active_mask[igrid] == 0)
								continue;
						for (ifunc = 0 ; ifunc < nfunc ; ifunc++){
							if (use_dx)
								v = ilevel * 1.0;
							else
								v = scal_data[igrid * twotondimMnfunc + icell * nfunc + ifunc];
							ray_length[iray * nfunc + ifunc] += cube_depth;
							if (max_op){
								if (v>maps[iray * nfunc + ifunc])
									maps[iray * nfunc + ifunc] = v;
							}
							else
								maps[iray * nfunc + ifunc] += v * cube_depth;
						}
					}
					else{ // # Go down the AMR tree
//						#########################################################################################
//						#                      Add the NECESSARY son cells to the pile                          # 
//						#########################################################################################
						if (ind_son1 != ind_son2){
							for (idim = 0 ; idim < ndim ; idim++){
								ind_entry = (ind_son1>>idim) & 1;
								ind_exit = (ind_son2>>idim) & 1;
								if (ind_entry==ind_exit){
									dim_constr[idim] = 1;
									dim_side[idim] = ind_entry;
								}
								else
									dim_constr[idim] = 0;
							}

//							# Add the POSSIBLY intersecting son cells different from the entry/exit point son cells
							for (ind_son = 0 ; ind_son < twotondim ; ind_son++){ // # All he son cells
								add = 1;
//								# Different from entry/exit point son cells
								if ((ind_son == ind_son1)||(ind_son == ind_son2))
									add = 0;
								else{
									for (idim = 0 ; idim < ndim ; idim++){
										if ((dim_constr[idim])&&(dim_side[idim] != ((ind_son>>idim) & 1))){
											add = 0;
											break;
										}
									}
								}

								if (add){
									head += 1;
									level_pile[head] = ilevel+1;
									igrid_pile[head] = igrid_son;
									icell_pile[head] = ind_son;
								}
							}

//							# If different from the son cell of the entry point, add the exit point son cell
							head+=1;
							level_pile[head] = ilevel+1;
							igrid_pile[head] = igrid_son;
							icell_pile[head] = ind_son2;
						}

//						# Add the son cell of the entry point of the ray in the father grid
						head+=1;
						level_pile[head] = ilevel+1;
						igrid_pile[head] = igrid_son;
						icell_pile[head] = ind_son1;
					}
				}
			}
		}
	}
	free(igrid_pile);
	free(icell_pile);
	free(level_pile);
	free(dim_constr);
	free(dim_side);
	free(block_center);
//	printf("maps[0] = %f \n",maps[0]);
}

